<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xhtml="http://www.w3.org/1999/xhtml"
	xmlns="http://www.w3.org/1999/xhtml"
>
	<xsl:output method="xml" encoding="UTF-8" />
	<!-- 
		Analyse a generic web page and return a vanilla-as-possible
		pure XHTML version.

		Embedded HTML should be character-escaped ...


		To find the nodes by local name without namespaces handicapping me,
		Input html must either have no namespace ( select="html" )
		or be in the xhtml namespace ( select="xhtml:html" ).
		Use these two patterns at all times when matching nodes by name :-(

		... This may be avoided slightly by adjusting the input namespace 
		during pre-processing ... although the syntax will still require 
		the prefix - a doc with any namespace requires the prefix to
		be used from then on.

		Due to namespaces again, if embedding unescaped html into the resulting xmlnode format,
		the nodes will be tagged with messy xmlns= tags. THIS IS CORRECT
		as we are not a true html document. BUT to turn this behaviour off,
		you can set xmlns="http://www.w3.org/1999/xhtml" in THIS file header.
		It's a lie, but it results in cleaner (albeit hybrid) code.
		
		The xsl:text nodes are just scattered around to force some pretty-printing on the output

		.dan.
	-->

	<!-- Caller can define the id of the 'content' block to be extracted -->
	<xsl:param name='contentid'>content</xsl:param>
	
	<xsl:template match="/">	
		<xsl:apply-templates select="html|xhtml:html" />
	</xsl:template>

	<xsl:template match="html|xhtml:html">	
		<html>
		<head>
		<!-- META And Header extraction -->
		
		<!-- 
			Note: TODO 
			Need to deal with the possible difference between 
			document title and H1 display title .
			Look for H1 or anything id=pagetitle first
			If that fails, use the meta title.
			Note, if the visible heading was found, it should be 
			removed from the body content so as not to show up again.
			Some exceptions to do so are defined later
		-->
	    <title>
			<xsl:call-template name="get-title" />
	    </title>

<xsl:text>
</xsl:text>

		<meta name="description">
			<xsl:attribute name="value">
				<xsl:call-template name="get-description" />
			</xsl:attribute>
		</meta>

		<!-- 
			Unsupported/unused  potential node attributes:
			nid="3" 
			promote="1" 
			moderate="0" 
  			changed="1127211405" 
  			created="1127194744" 
  			status="1"
		-->
		<xsl:for-each select="//head|//xhtml:head">
			<xsl:for-each select="meta|xhtml:meta|rel|xhtml:rel|style|xhtml:style">
	meta:			<xsl:copy><xsl:for-each select="@*"><xsl:copy></xsl:copy></xsl:for-each></xsl:copy>
			</xsl:for-each>
		</xsl:for-each>

		
		<!-- Content extraction -->
		<teaser>
		<!-- Grab the first two paragraphs ... found anywhere in the doc -->
		
			<!-- 
				Don't use just select="//p[position()=2]" as that gets all 
				p nodes who are number 2 in their respective contexts ... 
				I need to find numbers 1 and 2 in the set of all p nodes.
			-->
			<xsl:variable name="set" select="//p|//xhtml:p"/>
			<!-- <xsl:for-each select="$set[1]|$set[2]"> OK, just one para -->
			<xsl:for-each select="$set[1]">
						<xsl:apply-templates />
 			</xsl:for-each>

		</teaser>
		
		
		<vocabularies>
			<vocabulary vid="1">
				<name>Section</name>
				<terms>
						<term tid="3">
					<name>Portfolio</name>
					</term>
				</terms>
			</vocabulary>
		</vocabularies>
		

		</head>
<xsl:text>
</xsl:text>


<xsl:text>

</xsl:text>
		
		<body>
			<h1>
				<xsl:call-template name="get-title" />
			</h1>
			<div id="content">
				<xsl:call-template name="get-content" >
					<xsl:with-param name="contentid" select="$contentid" />
				</xsl:call-template >
			</div>
		</body>

		</html>
	</xsl:template>


	<xsl:template name="get-title">
		<!-- 
			Note: TODO 
			Need to deal with the possible difference between 
			document title and H1 display title .
			Look for H1 or anything id=pagetitle first
			If that fails, use the meta title.
			Some exceptions to do so are defined later
		-->
			<xsl:choose>
				
				<xsl:when test="//h1|//xhtml:h1">
			    	<xsl:value-of select="//h1|//xhtml:h1" />
				</xsl:when>
				
				<xsl:when test="//*[@id='pagetitle']">
					<!-- 
						Nice sites set the id of their title.
						May add other #ids here as I find them
					-->

			    	<xsl:value-of select="//*[@id='pagetitle']" />
				</xsl:when>

				<xsl:when test="count(//h2|//xhtml:h2)=1">
					<!-- 
						Maybe it used an h2 because h1 was ugly.
						If there is ONLY ONE h2, use that
					-->
			    	<xsl:value-of select="//h2|//xhtml:h2" />
				</xsl:when>
				
				<xsl:otherwise>
					<!-- 
						In practice, the html head title is often more verbose than we want
						and too often it's the same across whole sections, but it'll have to do.
					-->
			    	<xsl:value-of select="//*[name() = 'title']" />
				</xsl:otherwise>
				<!--
				  Any of these steps may fail if the heading contains only html, eg
				  <h1><img src='header.gif' /></h1>
				-->
			</xsl:choose>
	</xsl:template>


	<xsl:template name="get-content">
			<!--	
				Attempt to find any defined content divs. 
				A set of names is scanned for, and if any elements are 
				found with those ids or classes, that is used as the content.
				We go one-by-one through {user-defined contentid} followed
				by 'content', 'bodyCopy' and 'main'.
				If more than one match is found at the same level of precedence,
				the results are concatenated.
				
				Failing that, include everything found in the body.
			-->

			<xsl:param name="contentid"/>

		
			<xsl:choose>
				<!--
					Need to detect them before using them to avoid 
					duplication of content if more than one match is valid.
					So do a look-see test before select.
					Cannot group these tests, because we need to ensure 
					these checks happen in order of desired precedence.
					
					When we've found aour content, we discard its type and return its naked contents.
				-->

				<xsl:when test="//*[@id=$contentid or @class=$contentid ]">
					<!-- user-defined id tag -->
					 <xsl:comment>Imported From the element called <xsl:value-of select="$contentid"/></xsl:comment>
					<xsl:for-each select="//*[@id=$contentid or @class=$contentid]" >
						<xsl:apply-templates />
						<xsl:text>
						</xsl:text>
					</xsl:for-each>
				</xsl:when>

				<xsl:when test="//*[@id='main' or @class='main']">
					<xsl:for-each select="//*[@id='main' or @class='main']" >
						<xsl:apply-templates />
						<xsl:text>
						</xsl:text>
					</xsl:for-each>
				</xsl:when>

				<xsl:when test="//*[@id='content' or @class='content']">
					<xsl:for-each select="//*[@id='content']" >
						<xsl:apply-templates />
						<xsl:text>
						</xsl:text>
					</xsl:for-each>
				</xsl:when>

				<xsl:when test="//*[@id='bodyCopy']">
					<xsl:for-each select="//*[@id='bodyCopy']" >
						<xsl:apply-templates />
						<xsl:text>
						</xsl:text>
					</xsl:for-each>
				</xsl:when>

			  
				<xsl:when test="//*[@class='reference']">
				<!--
					This one grabs content from php.net documentation
				-->
					<xsl:for-each select="//*[@class='reference']" >
						<xsl:apply-templates />
					</xsl:for-each>
				</xsl:when>

				<xsl:otherwise>
					<xsl:text>
									
					</xsl:text>
					<xsl:comment>Couldn't selectively extract content, Imported Full Body :(</xsl:comment>
					<xsl:text>
					
					</xsl:text>
					<!-- Absolutely no magic here - dump it ALL in -->
					<xsl:apply-templates select="//body/*|//xhtml:body/*"/>
					

				</xsl:otherwise>


			</xsl:choose>
	</xsl:template>
	
	<xsl:template name="get-description">
		<!-- 
			Look for 
			a defined meta tag, 
			a 'description' id, 
			a 'description' class, 
			or the first paragraph. 
			In that order.
			Description text is a flat string with no formatting!
		-->
		<xsl:choose>
			<xsl:when test="//meta[@name='description']">
		    	<xsl:value-of select="//meta[@name='description']" />
			</xsl:when>
			<xsl:when test="//*[@id='description']">
		    	<xsl:value-of select="//*[@id='description']" />
			</xsl:when>
			<xsl:when test="//*[@class='description']">
		    	<xsl:value-of select="//*[@class='description']" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="set" select="//p|//xhtml:p"/>
				<xsl:for-each select="$set[1]">
							<xsl:apply-templates />
 				</xsl:for-each>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<!-- Identity transformation template - to inline existing HTML  -->			
	<xsl:template name="passthrough" match="@*|node()|text()">
	  <xsl:copy>
		<xsl:apply-templates select="@*|node()|text()"/>
	  </xsl:copy>
	</xsl:template>





</xsl:stylesheet>